
from string import *
from basics import readtable
from Numeric import *
import galphot

MAXGAL = 1
MAXLINE = 128
MAXHARM = 9
VALBAD = -33000.0

class GalPhot:
    def __init__(self):
        self.image = None
        self.xcen = None
        self.ycen = None
        self.ellipsefit = 1
        self.tabellipfit=0
        self.justres = 0
        self.rmin = 0.3
        self.rmax = 60.0
        self.radfac = 1.1
        self.linear = 0
        self.rshap = 0.0
        self.rcen = 0.0
        self.errshap = 0.5
        self.errcen = 0.1
        self.iter1 = 3
        self.iter2 = 3
        self.hmax = 6
        self.dposmax = 0.1
        self.dellmax = 0.1
        self.dangmax = 0.1
        self.r1 = 20.0
        self.r2 = 0.0
        self.nsammax = 200
        self.fracmin = 0.4
        self.cliplow = 0.0
        self.cliphigh = 0.0
        self.debug = 0
        self.extend = 1
        self.outfill = 1
        self.npolres = 4
    def initparams(self):
        self.ellipsefit = 1
        self.tabellipfit = 0
        self.justres = 0
        self.f0 = zeros((MAXLINE,4),Float32)
        self.i0 = zeros((MAXLINE,2),Float32)
        self.s0 = zeros((MAXLINE,2),Float32)
        self.x0 = zeros((MAXLINE,2),Float32)
        self.y0 = zeros((MAXLINE,2),Float32)
        self.r0 = zeros((MAXLINE,2),Float32)
        self.e0 = zeros((MAXLINE,2),Float32)
        self.t0 = zeros((MAXLINE,2),Float32)
        self.h0 = zeros((MAXLINE,MAXHARM,2,2),Float32)
        self.ci0 = zeros((MAXLINE,2),Float32)
        self.el0 = zeros((MAXLINE,2),Float32)
    def inputparams(self,gp):
        nl = gp.r.shape[0]
        ex = MAXLINE-nl
        self.initparams()
        self.f0 = concatenate([gp.f,zeros((ex,4),Float32)])
        self.i0 = concatenate([gp.i,zeros((ex,2),Float32)])
        self.s0 = concatenate([gp.s,zeros((ex,2),Float32)])
        self.x0 = concatenate([gp.x,zeros((ex,2),Float32)])
        self.y0 = concatenate([gp.y,zeros((ex,2),Float32)])
        self.r0 = concatenate([gp.r,zeros((ex,2),Float32)])
        self.e0 = concatenate([gp.e,zeros((ex,2),Float32)])
        self.t0 = concatenate([gp.t,zeros((ex,2),Float32)])
        self.ci0 = concatenate([gp.ci,zeros((ex,2),Float32)])
        self.el0 = concatenate([gp.el,zeros((ex,2),Float32)])
        self.h0 = concatenate([gp.h,zeros((nl,MAXHARM-gp.hmax,2,2),Float32)],1).astype(Float32)
        self.h0 = concatenate([self.h0,zeros((ex,MAXHARM,2,2),Float32)])
        self.tabellipfit = gp.ellipsefit
        self.justres = gp.justres
    def photfit(self):
       if self.image and self.xcen and self.ycen:
           self.resim,self.status = galphot.cphotfit( self.image,
              self.xcen,self.ycen,self.ellipsefit,
              self.rmin, self.rmax, self.radfac, self.linear,
              self.rshap, self.rcen, self.errshap, self.errcen,
              self.iter1, self.iter2, self.hmax, self.dposmax,
              self.dellmax, self.dangmax, self.r1, self.r2,
              self.nsammax, self.fracmin, self.cliplow, self.cliphigh,
              self.debug, self.extend, self.outfill, self.npolres,
              self.tabellipfit, self.justres,
              self.f0, self.i0, self.s0, self.x0, self.y0,
              self.r0, self.e0, self.t0, self.h0, self.ci0, self.el0)
           use = greater(self.r0[::,0],0)
           self.i,self.s,self.x,self.y,self.r,self.e,self.t,self.ci,self.el = \
             compress(use, [self.i0,self.s0,\
             self.x0,self.y0,self.r0,self.e0,self.t0,self.ci0,self.el0],1)
           self.f = compress(use,self.f0,0)
           self.h = compress(use,self.h0,0)[::,:self.hmax,::,::]
           return self.status
       else:
         print "Error."; return -200
    def writetable(self,filename):
       fu = open(filename,"w")
       print >> fu, "# r i di e de t dt x dx y dy s ds f1 f2 f3 f4",
       for k in range(self.hmax): print >> fu, "h"+str(k+1), "dh"+str(k+1),
       print >> fu
       for j in range(len(self.r)):
          print >> fu, self.r[j][0], self.i[j][0], self.i[j][1], self.e[j][0], self.e[j][1],\
            self.t[j][0], self.t[j][1], self.x[j][0], self.x[j][1], self.y[j][0], self.y[j][1],\
            self.s[j][0], self.s[j][1], self.f[j][0], self.f[j][1], self.f[j][2], self.f[j][3],
          for k in range(self.hmax):
             print >> fu, self.h[j][k][0][0], self.h[j][k][0][1],
          print >> fu
       fu.close()
    def readtable(self,filename):
       self.initparams()
       data = readtable(filename,header=0)
       self.hmax = count(join(data.keys()),"dh")
       nl = data["r"].shape[0]
       ex = MAXLINE-nl
       self.r = transpose([data["r"], 0*data["r"]])
       self.i = transpose([data["i"], data["di"]])
       self.e = transpose([data["e"], data["de"]])
       self.t = transpose([data["t"], data["dt"]])
       self.x = transpose([data["x"], data["dx"]])
       self.y = transpose([data["y"], data["dy"]])
       self.s = transpose([data["s"], data["ds"]])
       self.f = transpose([data["f1"], data["f2"], data["f3"], data["f4"]])
       self.ci = zeros((nl,2),Float32)
       self.el = zeros((nl,2),Float32)
       self.h1 = []
       for k in range(self.hmax): self.h1 += [data["h"+str(k+1)]]
       self.h2 = []
       for k in range(self.hmax): self.h2 += [data["dh"+str(k+1)]]
       self.h = asarray([self.h1,self.h2])
       self.h = reshape(concatenate([[self.h],[zeros(self.h.shape)]]),(nl,self.hmax,2,2))
       self.f0 = concatenate([self.f,zeros((ex,4),Float32)]).astype(Float32)
       self.r0 = concatenate([self.r,zeros((ex,2),Float32)]).astype(Float32)
       self.i0 = concatenate([self.i,zeros((ex,2),Float32)]).astype(Float32)
       self.s0 = concatenate([self.s,zeros((ex,2),Float32)]).astype(Float32)
       self.x0 = concatenate([self.x,zeros((ex,2),Float32)]).astype(Float32)
       self.y0 = concatenate([self.y,zeros((ex,2),Float32)]).astype(Float32)
       self.r0 = concatenate([self.r,zeros((ex,2),Float32)]).astype(Float32)
       self.e0 = concatenate([self.e,zeros((ex,2),Float32)]).astype(Float32)
       self.t0 = concatenate([self.t,zeros((ex,2),Float32)]).astype(Float32)
       self.h0 = concatenate([self.h,zeros((nl,MAXHARM-self.hmax,2,2),Float32)],1).astype(Float32)
       self.h0 = concatenate([self.h0,zeros((ex,MAXHARM,2,2),Float32)]).astype(Float32)
       self.tabellipfit = 1
       self.ellipsefit = 1
       self.justres = 0


