      subroutine model(nx,ny,pix,ngal,nl,inten,s,x,y,r,e,t,h,nhmax,ldif,
     &     dif,ellipfit,ierr)

c     subroutine calculates the pixel values as defined by r,e,t,i,h
c     several call to deeper subroutines which do the actual work


      include 'param.inc'
      include 'declare.inc'
      
      real	pix(nx,ny),dif(nx,ny)
      integer nl(ngal),nhmax
      logical	ldif,ellipfit
      
      real	hc(MAXHARM,MAXLINE),hs(MAXHARM,MAXLINE)
      real	st(MAXHARM),ct(MAXHARM)
      real	cxx(MAXLINE),cxy(MAXLINE),cyy(MAXLINE)
      
      logical out,circle
      character*128	string
      
      call startsub('MODEL')
      
      do 1  igal=1,ngal
         
         if (debug.ge.2) then
            write(string,'('' MODEL: initialize nhmax='',i5)') nhmax
            call gwrits(string,1)
         end if
         call modelinit(igal,nl(igal),inten,s,r,e,t,h,cxx,cxy,cyy,
     &        nhmax,hc,hs,circle,ierr)
         if (ierr.ne.0) return
         
         do 20 iy=1,ny
            do 21 ix=1,nx
               xi=float(ix)
               yi=float(iy)
               
               if (.not.circle) then
                  call modelvalue(xi,yi,igal,nl(igal),x,y,r,inten,
     &                 cxx,cxy,cyy,nhmax,hc,hs,pixval,ierr)
               else
                  call harmvalue(xi,yi,igal,nl(igal),x,y,r,inten,
     &                 cxx,cxy,cyy,nhmax,hc,hs,pixval,ierr)
               end if
               if (ierr.ne.0) return
               
               if (debug.ge.12) then
		  write(string,'(2i8,10g8.2)') ix, iy, xi, yi, pixval
		  call gwrits(string,1)
               end if
               
               if (igal.eq.1) then
                  if (ldif) then
                     if (pix(ix,iy).gt.VALMIN.and.pixval.gt.VALMIN) then
                        dif(ix,iy)=pix(ix,iy)-pixval
                     else
                        dif(ix,iy)=VALBAD
                     end if
                  else
                     dif(ix,iy)=pixval
                  end if
               else
                  if (dif(ix,iy).gt.VALMIN) then
                     if (ldif) then
                        dif(ix,iy)=dif(ix,iy)-pixval
                     else
                        dif(ix,iy)=dif(ix,iy)+pixval
                     end if
                  end if
               end if
 21         continue
 20      continue
         
 1    continue

      ierr=0
      return
      end
      
      subroutine modelinit(igal,nl,inten,s,r,e,t,h,
     &     cxx,cxy,cyy,nhmax,hc,hs,circle,ierr)
      
c     subroutine initializes values for model. 
      
      include 'param.inc'
      include 'declare.inc'
      
      real	hc(MAXHARM,MAXLINE),hs(MAXHARM,MAXLINE)
      real	st(MAXHARM),ct(MAXHARM)
      real	cxx(MAXLINE),cxy(MAXLINE),cyy(MAXLINE)
      logical   circle
      
      character*128	string
      
      call startsub('MODELINIT')
      
      circle=.true.
      do 10 il=1,nl
         st(1)=sin(t(1,il,igal))
         ct(1)=cos(t(1,il,igal))
         circle=circle.and.e(1,il,igal).eq.0.
         if (e(1,il,igal).gt.1.) then
            write(string,'('' MODELINIT: ERROR: ellip > 1 '')')
            call gwrits(string,2)
            ierr=1
            return
         end if
         if (r(1,il,igal).ne.0) then
            a=r(1,il,igal)/sqrt(1.-e(1,il,igal))
            b=sqrt(1.-e(1,il,igal))*r(1,il,igal)
            cxx(il)=ct(1)*ct(1)/(a*a)+st(1)*st(1)/(b*b)
            cxy(il)=2.*ct(1)*st(1)*(1./(a*a)-1./(b*b))
            cyy(il)=st(1)*st(1)/(a*a)+ct(1)*ct(1)/(b*b)
         else
            cxx(il)=1.e20
            cxy(il)=0.
            cyy(il)=1.e20
         end if
         
         if (il.gt.1) then
            delta=(cxx(il)-cxx(il-1))*(cyy(il)-cyy(il-1))-
     &           ((cxy(il)-cxy(il-1))**2)/4.
            if (delta.lt.0) then
               write(string,
     $              '('' MODELINIT: WARNING: ellipses overlapping'',
     $              ''at r='',f8.2)') r(1,il,igal)
               call gwrits(string,2)
            end if
         end if
         
         do 11 ih=2,nhmax
            st(ih)=st(ih-1)*ct(1)+st(1)*ct(ih-1)
            ct(ih)=ct(ih-1)*ct(1)-st(1)*st(ih-1)
 11      continue
         harfac=abs(r(1,il,igal)*s(1,il,igal))
         do 12 ih=1,nhmax
            hc(ih,il)=(-st(ih)*h(1,2,ih,il,igal)+
     &           ct(ih)*h(1,1,ih,il,igal) )*harfac
            hs(ih,il)=( st(ih)*h(1,1,ih,il,igal)+
     &           ct(ih)*h(1,2,ih,il,igal) )*harfac
 12      continue
         
         if (debug.ge.5) then
            write(string,'('' MODELINIT: '',10g10.3)')
     &           r(1,il,igal),inten(1,il,igal),cxx(il),cxy(il),cyy(il),
     &           harfac,hc(2,il),hs(2,il)
            call gwrits(string,1)
         end if
         
 10   continue

      ierr=0
      return
      end
      
      subroutine modelvalue(xi,yi,igal,nl,x,y,r,inten,cxx,cxy,cyy,
     &     nhmax,hc,hs,pixval,ierr)
      
c     subroutine calculates the actual value given by the model
c     at a certain xi & yi
      
      include 'param.inc'
      include 'declare.inc'
      
      real	hc(MAXHARM,MAXLINE),hs(MAXHARM,MAXLINE)
      real	st(MAXHARM),ct(MAXHARM)
      real	cxx(MAXLINE),cxy(MAXLINE),cyy(MAXLINE)
      
      real	rinv(10),intr(10)
      
c      static iv0
      logical	out
      character*128	string      
      
      if (nl.lt.npolres) then
         write(string,'('' MODELVALUE: ERROR: nl < npolres '')')
         call gwrits(string,2)
         ierr=1
         return
      end if

      hval = 0.0
      
      if (iv0.lt.1) iv0=nl/2
      if (iv0.ge.nl) iv0=nl-1
      iv1=iv0+1
      
      call findellipse(xi,yi,iv0,iv1,nl,igal,x,y,
     &     cxx,cxy,cyy,d0,d1,out,ierr)
      if (ierr.ne.0) return
      
      if (debug.ge.20) then
         write(string,'('' MODELVALUE: '',l10,2f8.2,2i8,2f8.2)') 
     $        out,xi,yi,iv0,iv1,d0,d1
         call gwrits(string,1)
      end if
      
      if (.not.out) then
         ipol0=iv0-(npolres/2-1)
         if (ipol0.lt.1) ipol0=1
         if (ipol0+npolres-1.gt.nl) ipol0=nl-(npolres-1)
         
         index=0
         do 10 i=ipol0,ipol0+npolres-1
            index=index+1
            if (i.eq.iv0) then
               rinv(index)=d0
            else if (i.eq.iv1) then
               rinv(index)=d1
            else
               rinv(index)=
     &              cxx(i)*(xi-x(1,i,igal))**2+cxy(i)*(xi-x(1,i,igal))*
     &              (yi-y(1,i,igal))+cyy(i)*(yi-y(1,i,igal))**2-1.
               
            end if
            if (index.eq.1.or.rinv(index).lt.rinv(index-1)) then
               intr(index)=inten(1,i,igal)
               
               if (r(1,i,igal).ne.0.) then
                  ct(1)=(xi-x(1,i,igal))/r(1,i,igal)
                  st(1)=(yi-y(1,i,igal))/r(1,i,igal)
               else
                  ct(1)=0.
                  st(1)=0.
               end if
               
               do 11 ih=2,nhmax
                  st(ih)=st(ih-1)*ct(1)+st(1)*ct(ih-1)
                  ct(ih)=ct(ih-1)*ct(1)-st(1)*st(ih-1)
 11            continue
               
               do 12 ih=2,nhmax
                  intr(index)=intr(index)+
     &                 st(ih)*hs(ih,i)+ct(ih)*hc(ih,i)
 12            continue
            else
c     correction for overlapping ellipses
c     if rinv > rinv(ind-1) the last point is omitted
               index=index-1
            end if
 10      continue
         
         if (debug.ge.12) then
            write(string,'(8f10.4)') (rinv(i),i=1,index)
            call gwrits(string,1)
            write(string,'(8g10.4)') (intr(i),i=1,index)
            call gwrits(string,1)
         end if
         
         rxy=0.
         call polintarray(rinv,intr, 1, index, rxy, value, error,ierr)
         if (ierr.ne.0) return
         
         pixval=value+hval
      else
         if (d0.lt.0.and.d1.lt.0.) then
            pixval=inten(1,1,igal)
         else
            if (outfill) then
               pixval=inten(1,nl,igal)
            else
               pixval=VALBAD
            end if
         end if
      end if
      
      if (debug.ge.20) then
         write(string,'('' MODELVALUE: '',8g10.4)') pixval
         call gwrits(string,1)
      end if
      
      ierr=0
      return
      end
      
      subroutine harmvalue(xi,yi,igal,nl,x,y,r,inten,cxx,cxy,cyy,
     &     nhmax,hc,hs,pixval,ierr)
      
c     subroutine calculates the model value at xi & yi for harmonical model
      
      include 'param.inc'
      include 'declare.inc'
      
      real	hc(MAXHARM,MAXLINE),hs(MAXHARM,MAXLINE)
      real	st(MAXHARM),ct(MAXHARM)
      real	cxx(MAXLINE),cxy(MAXLINE),cyy(MAXLINE)
      
      real	intray(10),rray(10)
      
      character*128	string
      
      if (nl.lt.npolres) then
         write(string,'('' MODELVALUE: ERROR: nl < npolres '')')
         call gwrits(string,2)
         ierr=1
         return
      end if
      
      if (iv0.lt.1) iv0=1
      if (iv0.ge.nl) iv0=nl-1
      iv1=iv0+1
      
      rad=sqrt((xi-x(1,1,igal))**2+(yi-y(1,1,igal))**2)
      
      if (rad.gt.r(1,nl,igal)) then
         
c     pixel lying outside of maximum radius
         
         pixval=inten(1,nl,igal)
      else  if (rad.lt.r(1,1,igal)) then
         
c     pixel lying outside of minimum radius
         
         pixval=inten(1,1,igal)
      else
         
         call huntarray(r(1,1,igal),2,nl,rad,iv0)
         iv1=iv0+1
         
         if (rad.ne.0.) then
            ct(1)=(xi-x(1,1,igal))/rad
            st(1)=(yi-y(1,1,igal))/rad
         else
            ct(1)=0.
            st(1)=0.
         end if
         
         do 10 ih=2,nhmax
            st(ih)=st(ih-1)*ct(1)+st(1)*ct(ih-1)
            ct(ih)=ct(ih-1)*ct(1)-st(1)*st(ih-1)
 10      continue
         
         hval=0.
         
         ipol0=iv0-(npolres/2-1)
         if (ipol0.lt.1) ipol0=1
         if (ipol0+npolres-1.gt.nl) ipol0=nl-(npolres-1)
         
         do 20 i=ipol0,ipol0+npolres-1
            index=i-(ipol0-1)
            rray(index)=r(1,i,igal)
            intray(index)=inten(1,i,igal)	
            do 21 ih=1,nhmax
               intray(index)=intray(index)+
     &              st(ih)*hs(ih,i)+ct(ih)*hc(ih,i)
 21         continue
 20      continue
         
         call polintarray(rray,intray, 1, npolres, rad, pixval, error,
     $        ierr)
         if (ierr.ne.0) return
      end if
      
      if (debug.ge.12) then
         write(string,'('' HARMVALUE: '',f8.2,i5,8g10.4)') 
     $        rad,iv0,value,hval,pixval
         call gwrits(string,1)
      end if
      
      ierr=0
      return
      end
      
      
      subroutine findellipse(xi,yi,iv0,iv1,nl,igal,x,y,
     &     cxx,cxy,cyy,d0,d1,out,ierr)
      
c     subroutine determines whithin which ellipses a point lies
      
      include 'param.inc'
      include 'declare.inc'
      
      real	cxx(MAXLINE),cxy(MAXLINE),cyy(MAXLINE)
      logical out
      
      
      if (iv0.lt.1) iv0=1
      if (iv0.gt.nl-1) iv0=nl-1
      iv1=iv0+1
      
      d1=cxx(iv1)*(xi-x(1,iv1,igal))**2+cxy(iv1)*(xi-x(1,iv1,igal))*
     &     (yi-y(1,iv1,igal))+cyy(iv1)*(yi-y(1,iv1,igal))**2-1.
      if (iv1.eq.nl.and.d1.gt.0.) then
c     point outside outer ellipse
         out=.true.
         ierr=0
         return
      end if
      
      d0=cxx(iv0)*(xi-x(1,iv0,igal))**2+cxy(iv0)*(xi-x(1,iv0,igal))*
     &     (yi-y(1,iv0,igal))+cyy(iv0)*(yi-y(1,iv0,igal))**2-1.
      
 10   if ((d0.lt.0..and.iv0.gt.1).or.(d1.gt.0..and.iv1.lt.nl)) then
         if (d1.gt.0.) then
	    iv0=iv1
	    iv1=iv1+1
	    d0=d1
            
	    d1=cxx(iv1)*(xi-x(1,iv1,igal))**2+
     $           cxy(iv1)*(xi-x(1,iv1,igal))*
     &           (yi-y(1,iv1,igal))+cyy(iv1)*(yi-y(1,iv1,igal))**2-1.
         else
	    iv1=iv0
	    iv0=iv0-1
	    d1=d0
	    d0=cxx(iv0)*(xi-x(1,iv0,igal))**2+
     $           cxy(iv0)*(xi-x(1,iv0,igal))*
     &           (yi-y(1,iv0,igal))+cyy(iv0)*(yi-y(1,iv0,igal))**2-1.
         end if
         goto 10
      end if
      
      out=(d1.gt.0.).or.(d0.lt.0.)

c     if out=true then point either inside inner ellipse or outside
c     outer ellipse

      
      ierr=0
      return
      end

